@@ -19,8 +19,10 @@
  * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
+#include <float.h>
 #include <handy.h>
 #include <glib/gi18n.h>
+#include <float.h>
 #include <math.h>
 #include "list-box-helper.h"
 #include "cc-display-settings.h"
@@ -42,12 +44,14 @@ struct _CcDisplaySettings
   GListStore       *orientation_list;
   GListStore       *refresh_rate_list;
   GListStore       *resolution_list;
+  GListStore       *scale_list;
 
   GtkWidget        *orientation_row;
   GtkWidget        *refresh_rate_row;
   GtkWidget        *resolution_row;
   GtkWidget        *scale_bbox;
-  GtkWidget        *scale_row;
+  GtkWidget        *scale_buttons_row;
+  GtkWidget        *scale_combo_row;
   GtkWidget        *underscanning_row;
   GtkWidget        *underscanning_switch;
 };
@@ -208,11 +212,12 @@ sort_modes_by_area_desc (CcDisplayMode *a, CcDisplayMode *b)
   cc_display_mode_get_resolution (a, &wa, &ha);
   cc_display_mode_get_resolution (b, &wb, &hb);
 
-  /* Prefer wide screen if the size is equal */
-  res = wb*hb - wa*ha;
-  if (res == 0)
-    return wb - wa;
-  return res;
+  /* Sort first by width, then height.
+   * We used to sort by area, but that can be confusing. */
+  res = wb - wa;
+  if (res)
+    return res;
+  return hb - ha;
 }
 
 static gint
@@ -230,8 +235,8 @@ cc_display_settings_rebuild_ui (CcDisplaySettings *self)
   gint width, height;
   CcDisplayMode *current_mode;
   GtkRadioButton *group = NULL;
-  gint buttons = 0;
-  const gdouble *scales, *scale;
+  g_autoptr(GArray) scales = NULL;
+  gint i;
 
   self->idle_udpate_id = 0;
 
@@ -240,7 +245,8 @@ cc_display_settings_rebuild_ui (CcDisplaySettings *self)
       gtk_widget_set_visible (self->orientation_row, FALSE);
       gtk_widget_set_visible (self->refresh_rate_row, FALSE);
       gtk_widget_set_visible (self->resolution_row, FALSE);
-      gtk_widget_set_visible (self->scale_row, FALSE);
+      gtk_widget_set_visible (self->scale_combo_row, FALSE);
+      gtk_widget_set_visible (self->scale_buttons_row, FALSE);
       gtk_widget_set_visible (self->underscanning_row, FALSE);
 
       return G_SOURCE_REMOVE;
@@ -249,6 +255,7 @@ cc_display_settings_rebuild_ui (CcDisplaySettings *self)
   g_object_freeze_notify ((GObject*) self->orientation_row);
   g_object_freeze_notify ((GObject*) self->refresh_rate_row);
   g_object_freeze_notify ((GObject*) self->resolution_row);
+  g_object_freeze_notify ((GObject*) self->scale_combo_row);
   g_object_freeze_notify ((GObject*) self->underscanning_switch);
 
   cc_display_monitor_get_geometry (self->selected_output, NULL, NULL, &width, &height);
@@ -355,10 +362,6 @@ cc_display_settings_rebuild_ui (CcDisplaySettings *self)
       gint w, h;
       CcDisplayMode *mode = CC_DISPLAY_MODE (item->data);
 
-      /* Exclude unusable low resolutions */
-      if (!cc_display_config_is_scaled_mode_valid (self->config, mode, 1.0))
-        continue;
-
       cc_display_mode_get_resolution (mode, &w, &h);
 
       /* Find the appropriate insertion point. */
@@ -389,44 +392,58 @@ cc_display_settings_rebuild_ui (CcDisplaySettings *self)
 
   /* Scale row is usually shown. */
   gtk_container_foreach (GTK_CONTAINER (self->scale_bbox), (GtkCallback) gtk_widget_destroy, NULL);
+  g_list_store_remove_all (self->scale_list);
+  gtk_widget_set_visible (self->scale_buttons_row, FALSE);
+  gtk_widget_set_visible (self->scale_combo_row, FALSE);
   scales = cc_display_mode_get_supported_scales (current_mode);
-  for (scale = scales; *scale != 0.0; scale++)
+  for (i = 0; i < scales->len; i++)
     {
       g_autofree gchar *scale_str = NULL;
-      GtkWidget *scale_btn;
+      double scale = g_array_index (scales, double, i);
+      gboolean is_selected;
 
-      if (!cc_display_config_is_scaled_mode_valid (self->config,
-                                                   current_mode,
-                                                   *scale) &&
-          cc_display_monitor_get_scale (self->selected_output) != *scale)
-        continue;
+      scale_str = make_scale_string (scale);
+      is_selected = G_APPROX_VALUE (cc_display_monitor_get_scale (self->selected_output),
+                                    scale, DBL_EPSILON);
 
-      scale_str = make_scale_string (*scale);
+      if (scales->len > MAX_SCALE_BUTTONS)
+        {
+          g_autoptr(HdyValueObject) value_object = NULL;
 
-      scale_btn = gtk_radio_button_new_with_label_from_widget (group, scale_str);
-      if (!group)
-        group = GTK_RADIO_BUTTON (scale_btn);
-      gtk_toggle_button_set_mode (GTK_TOGGLE_BUTTON (scale_btn), FALSE);
-      g_object_set_data_full (G_OBJECT (scale_btn),
-                              "scale",
-                              g_memdup (scale, sizeof (gdouble)),
-                              g_free);
-      gtk_widget_show (scale_btn);
-      gtk_container_add (GTK_CONTAINER (self->scale_bbox), scale_btn);
-      g_signal_connect_object (scale_btn,
-                               "notify::active",
-                               G_CALLBACK (on_scale_btn_active_changed_cb),
-                               self, 0);
+          value_object = hdy_value_object_new_collect (G_TYPE_STRING, scale_str);
+          g_list_store_append (self->scale_list, value_object);
+          g_object_set_data_full (G_OBJECT (value_object), "scale",
+                                  g_memdup2 (&scale, sizeof (double)), g_free);
+          if (is_selected)
+            hdy_combo_row_set_selected_index (HDY_COMBO_ROW (self->scale_combo_row),
+                                              g_list_model_get_n_items (G_LIST_MODEL (self->scale_list)) - 1);
+        }
+      else
+        {
+          GtkWidget *scale_btn = gtk_radio_button_new_with_label_from_widget (group, scale_str);
+          g_object_set_data_full (G_OBJECT (scale_btn), "scale",
+                                  g_memdup2 (&scale, sizeof (double)), g_free);
 
-      if (cc_display_monitor_get_scale (self->selected_output) == *scale)
-        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (scale_btn), TRUE);
+          if (!group)
+            group = GTK_RADIO_BUTTON (scale_btn);
+          gtk_toggle_button_set_mode (GTK_TOGGLE_BUTTON (scale_btn), FALSE);
+          gtk_widget_show (scale_btn);
+          gtk_container_add (GTK_CONTAINER (self->scale_bbox), scale_btn);
+          /* Set active before connecting the signal */
+          if (is_selected)
+            gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (scale_btn), TRUE);
 
-      buttons += 1;
-      if (buttons >= MAX_SCALE_BUTTONS)
-        break;
+          g_signal_connect_object (scale_btn,
+                                   "notify::active",
+                                   G_CALLBACK (on_scale_btn_active_changed_cb),
+                                   self, 0);
+        }
     }
 
-  gtk_widget_set_visible (self->scale_row, buttons > 1);
+  if (scales->len > MAX_SCALE_BUTTONS)
+    gtk_widget_set_visible (self->scale_combo_row, TRUE);
+  else
+    gtk_widget_set_visible (self->scale_buttons_row, scales->len > 1);
 
   gtk_widget_set_visible (self->underscanning_row,
                           cc_display_monitor_supports_underscanning (self->selected_output) &&
@@ -438,6 +455,7 @@ cc_display_settings_rebuild_ui (CcDisplaySettings *self)
   g_object_thaw_notify ((GObject*) self->orientation_row);
   g_object_thaw_notify ((GObject*) self->refresh_rate_row);
   g_object_thaw_notify ((GObject*) self->resolution_row);
+  g_object_thaw_notify ((GObject*) self->scale_combo_row);
   g_object_thaw_notify ((GObject*) self->underscanning_switch);
   self->updating = FALSE;
 
@@ -539,6 +557,27 @@ on_scale_btn_active_changed_cb (GtkWidget         *widget,
   g_signal_emit_by_name (G_OBJECT (self), "updated", self->selected_output);
 }
 
+static void
+on_scale_selection_changed_cb (GtkWidget         *widget,
+                               GParamSpec        *pspec,
+                               CcDisplaySettings *self)
+{
+  int idx;
+  double scale;
+  g_autoptr(HdyValueObject) obj = NULL;
+
+  if (self->updating)
+    return;
+
+  idx = hdy_combo_row_get_selected_index (HDY_COMBO_ROW (self->scale_combo_row));
+  obj = g_list_model_get_item (G_LIST_MODEL (self->scale_list), idx);
+  scale = *(gdouble*) g_object_get_data (G_OBJECT (obj), "scale");
+
+  cc_display_monitor_set_scale (self->selected_output, scale);
+
+  g_signal_emit_by_name (G_OBJECT (self), "updated", self->selected_output);
+}
+
 static void
 on_underscanning_switch_active_changed_cb (GtkWidget         *widget,
                                            GParamSpec        *pspec,
@@ -617,6 +656,7 @@ cc_display_settings_finalize (GObject *object)
   g_clear_object (&self->orientation_list);
   g_clear_object (&self->refresh_rate_list);
   g_clear_object (&self->resolution_list);
+  g_clear_object (&self->scale_list);
 
   if (self->idle_udpate_id)
     g_source_remove (self->idle_udpate_id);
@@ -669,13 +709,15 @@ cc_display_settings_class_init (CcDisplaySettingsClass *klass)
   gtk_widget_class_bind_template_child (widget_class, CcDisplaySettings, refresh_rate_row);
   gtk_widget_class_bind_template_child (widget_class, CcDisplaySettings, resolution_row);
   gtk_widget_class_bind_template_child (widget_class, CcDisplaySettings, scale_bbox);
-  gtk_widget_class_bind_template_child (widget_class, CcDisplaySettings, scale_row);
+  gtk_widget_class_bind_template_child (widget_class, CcDisplaySettings, scale_buttons_row);
+  gtk_widget_class_bind_template_child (widget_class, CcDisplaySettings, scale_combo_row);
   gtk_widget_class_bind_template_child (widget_class, CcDisplaySettings, underscanning_row);
   gtk_widget_class_bind_template_child (widget_class, CcDisplaySettings, underscanning_switch);
 
   gtk_widget_class_bind_template_callback (widget_class, on_orientation_selection_changed_cb);
   gtk_widget_class_bind_template_callback (widget_class, on_refresh_rate_selection_changed_cb);
   gtk_widget_class_bind_template_callback (widget_class, on_resolution_selection_changed_cb);
+  gtk_widget_class_bind_template_callback (widget_class, on_scale_selection_changed_cb);
   gtk_widget_class_bind_template_callback (widget_class, on_underscanning_switch_active_changed_cb);
 }
 
@@ -691,6 +733,7 @@ cc_display_settings_init (CcDisplaySettings *self)
   self->orientation_list = g_list_store_new (HDY_TYPE_VALUE_OBJECT);
   self->refresh_rate_list = g_list_store_new (CC_TYPE_DISPLAY_MODE);
   self->resolution_list = g_list_store_new (CC_TYPE_DISPLAY_MODE);
+  self->scale_list = g_list_store_new (HDY_TYPE_VALUE_OBJECT);
 
   self->updating = TRUE;
 
@@ -706,6 +749,10 @@ cc_display_settings_init (CcDisplaySettings *self)
                                  G_LIST_MODEL (self->resolution_list),
                                  (HdyComboRowGetNameFunc) make_resolution_string,
                                  NULL, NULL);
+  hdy_combo_row_bind_name_model (HDY_COMBO_ROW (self->scale_combo_row),
+                                 G_LIST_MODEL (self->scale_list),
+                                 (HdyComboRowGetNameFunc) hdy_value_object_dup_string,
+                                 NULL, NULL);
 
   self->updating = FALSE;
 }
